#include <stdio.h>
#include <string.h>
#include "sap_moni.h"
#include "sap_moni_ccm.h"


#define ERROR 2 
#define TEMPLATE_SAP_SYSTEMS "sap_systems"

/*=======================================================================
  (C) Copyright SAP AG, 2002

         
 Author  : Dostert/SAP
 Project : Get data from R/3 using XMB
 Using   : RFCSDK 
 ========================================================================*/

void usage(char *progname) {
        #define BL printf(



         BL"Agent\n\n");
         BL"Syntax: %s <Instance> <RFC-Template>\n\n",  progname     );


         BL"       <RFC-Template> defined in %s\n", AGENT_LOGIN_FILE  );
}


int main(int argc, char **argv) {
	int i,j;

	RFC_HANDLE rfc_handle;
	AlMoniHandle_T *MoniHandle; 
	int   NrOfMoniTempl;

	AlMoniTemplate_T *moni_templ; 
	BAPITNDEXT *tTreeNodes;
	
	AlMoniNode_T *nodes;
        unsigned int NrOfNodes;	

	int   highest_alert = 0;
	char  *bg_color;

	char tid_c[85];
	
	BAPITID    tid;
	BAPIPAPROP Properties;

	if (argc < 3) {
		usage(argv[0]);
		return ERROR;
	}

	if ( sapmon_init(&MoniHandle, &NrOfMoniTempl) >=  AG_FIRST_ERROR) {
		printf("Could not initialize INI-File\n");
		return ERROR;
	}

	if( sapmon_connect(argv[2], &rfc_handle) >=  AG_FIRST_ERROR) {
		printf("Could not log on\n");
		return ERROR;
	}

	
	for (i = 0; i <  NrOfMoniTempl; i++) {
		if ( strcmp(sapmon_get_template_name(MoniHandle[i]), TEMPLATE_SAP_SYSTEMS) != 0) 
			continue;

		moni_templ = get_template_by_handle(MoniHandle[i]);
		sprintf(moni_templ->Pattern[0], "*\\Availability\\%s\\Dialog Response Time", argv[1]);
		sprintf(moni_templ->Pattern[1], "*\\Availability\\%s\\Users Logged On", argv[1]);
		moni_templ->NumberOfPatterns = 2; 
		
		sapmon_request(&rfc_handle, &(MoniHandle[i]), 1 );
		sapmon_request_tree(MoniHandle[i], &nodes, &NrOfNodes);	
	
		if (NrOfNodes > 0) 
			printf("<table>");

		for (j = 0; j < NrOfNodes; j++) {
			if (nodes[j].Actualval > highest_alert)
				highest_alert = nodes[j].Actualval;			
			switch(nodes[j].Actualval) {
				case 1: 
				   bg_color = "OK";
				   break;
				case 2: 
			           bg_color = "WARNING";
				   break;
				case 3: 
		                   bg_color = "CRITICAL";
				   break;
				default:
				   bg_color = "UNKNOWN";
				   break;
			}						
			
			tTreeNodes = ItGetLine( moni_templ->TreeNodes,j+1);	
			if( strncmp(tTreeNodes->Mtclass,  MT_CLASS_PERFORMANCE, 3) == 0) {
				memset(&tid, ' ', sizeof(tid));

			        strncpy(tid.Mtsysid, tTreeNodes->Mtsysid, 8);
         			strncpy(tid.Mtmcname, tTreeNodes->Mtmcname, 40);
         			strncpy(tid.Mtnumrange, tTreeNodes->Mtnumrange, 3);
         			strncpy(tid.Mtuid, tTreeNodes->Mtuid, 10);
         			strncpy(tid.Mtclass, tTreeNodes->Mtclass,3);
         			strncpy(tid.Mtindex, tTreeNodes->Mtindex, 10);
         			strncpy(tid.Extindex, tTreeNodes->Extindex, 10);
				
				bapi_system_mte_getperfprop(&rfc_handle, &tid, &Properties);

				strncpy(tid_c, (char *)&tid,84);
				tid_c[84] = '\0';

				printf("<tr><td CLASS='status%s'><a href='cmd.cgi?cmd_typ=91&TID=%s&thr_g2y=%d&thr_r2y=%d&thr_y2g=%d&thr_y2r=%d&unit=%s&rfc_template=%s'>%s %s</a></td></tr>", 
					bg_color, 
					url_encode(tid_c),  
					Properties.Treshg2y, Properties.Treshr2y,
					Properties.Treshy2g, Properties.Treshy2r,
					url_encode(Properties.Valunit),
					url_encode(argv[2]),
					nodes[j].Mtnameshrt, nodes[j].currentValue);
			} else {
				printf("<tr><td CLASS='status%s'>%s %s</td></tr>", bg_color, nodes[j].Mtnameshrt, nodes[j].currentValue);
			}


		}
			
		if (NrOfNodes > 0)
                        printf("</table>");
	}

        //Nothing found
        if (NrOfNodes ==  0) {
 	      highest_alert = 3;
              printf("No information gathered! System up?\n");
        } 
		

	//Log off
	sapmon_disconnect(&rfc_handle);	
	
	return highest_alert -1;
}
